open Escape

let xml_escape_char = function
  | '&' -> "&amp;"
  | '<' -> "&lt;"
  | '>' -> "&gt;"
  | '"' -> "&quot;"
  | '\'' -> "&apos;"
  | '\n' -> "&#xA;"
  | '\r' -> "&#xD;"
  | '\t' -> "&#x9;"
  | ' ' .. '~' as printable -> String.make 1 printable
  | unprintable ->
      "<char>" ^ string_of_int (int_of_char unprintable) ^ "</char>"

let xml_escape_string str =
  let length = String.length str in
  let buffer = Buffer.create length in
  for index = 0 to length - 1 do
    Buffer.add_string buffer (xml_escape_char (String.get str index))
  done;
  Buffer.contents buffer

let xml_escape_wchar =
  let limit upper probe = (Int64.to_float (Int64.sub upper probe)) > 0.5 in
  let fits_byte = limit (Int64.of_int 0x100) in
  let fits_universal_8 = limit (Int64.of_string "0x100000000") in
  (fun charcode ->
    if fits_byte charcode then
      escape_char (Char.chr (Int64.to_int charcode))
    else if fits_universal_8 charcode then
      Printf.sprintf "&#x%Lx;" charcode
    else
      (* no idea what this means *)
      invalid_arg "Cprint.escape_string_intlist")

let xml_escape_wstring (str : int64 list) =
  let length = List.length str in
  let buffer = Buffer.create length in
  let append charcode =
    let addition = xml_escape_wchar charcode in
    Buffer.add_string buffer addition
  in
  List.iter append str;
  Buffer.contents buffer
