open Pretty

(* indentation level *)
let depth = 2

(* turn a (key, value) list into " key1="value1" key2="value2" ..." *)
let rec mk_attr_str = function
  | [] -> ""
  | (k, v) :: t -> " " ^ k ^ "=\"" ^ v ^ "\"" ^ mk_attr_str t

(* concatenate together all the docs in a doc list *)
let rec cat = function
  | [] -> nil
  | h :: t -> h ++ (cat t)

(* tell whether a doc, when actually printed, will just be empty *)
let empty d = (sprint ~width:max_int d) = ""

let xml_simple name attrs child_lst =
  let children = cat child_lst in
  if name = "" then
    nil
  else if empty children then
    text ("<" ^ name ^ mk_attr_str attrs ^ "/>") ++ line
  else
    text ("<" ^ name ^ mk_attr_str attrs ^ ">")
      ++ children
      ++ text ("</" ^ name ^ ">")
      ++ line

let xml_complex name attrs child_lst =
  let children = cat child_lst in
  if name = "" then
    nil
  else if empty children then
    text ("<" ^ name ^ mk_attr_str attrs ^ "/>") ++ line
  else
    text ("<" ^ name ^ mk_attr_str attrs ^ ">")
      ++ line
      ++ indent depth children
      ++ text ("</" ^ name ^ ">")
      ++ line

let rec xml_seq wrapper f = function
  | [] -> nil
  | h :: t -> wrapper [f h] ++ xml_seq wrapper f t
